// Force strict mode and setup for ESM
"use strict";
import {
  init_esbuild_shims
} from "./chunk-A4BMJM77.js";

// packages/acp-bridge/src/bridgeTypes.ts
init_esbuild_shims();
var MID_TURN_QUEUE_DRAIN_METHOD = "craft/drainMidTurnQueue";
var CLIENT_MCP_OVER_WS_CONFIG_FLAG = "__clientMcpOverWs";

export {
  MID_TURN_QUEUE_DRAIN_METHOD,
  CLIENT_MCP_OVER_WS_CONFIG_FLAG
};
/**
 * @license
 * Copyright 2025 Qwen Team
 * SPDX-License-Identifier: Apache-2.0
 */
