/**
 * @license
 * Copyright 2026 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
// If moved update release-please config
// x-release-please-start-version
export const VERSION = '1.4.0';
// x-release-please-end
//# sourceMappingURL=version.js.map