/**
 * Copyright 2022 Google LLC.
 * Copyright (c) Microsoft Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
import { type LoggerFn } from './log.js';
import type { Result } from './result.js';
export declare class ProcessingQueue<T> {
    #private;
    static readonly LOGGER_PREFIX: "debug:queue";
    constructor(processor: (arg: T) => Promise<void>, logger?: LoggerFn);
    add(entry: Promise<Result<T>>, name: string): void;
}
