// Force strict mode and setup for ESM
"use strict";
import {
  WorkspacePermissionRulesSessionRequiredError,
  WorkspaceSettingsPartialPersistError
} from "./chunk-BMFMJINR.js";
import "./chunk-A4BMJM77.js";
import "./chunk-J2S4EL5Y.js";
export {
  WorkspacePermissionRulesSessionRequiredError,
  WorkspaceSettingsPartialPersistError
};
