// Force strict mode and setup for ESM
"use strict";
import {
  init_esbuild_shims
} from "./chunk-A4BMJM77.js";

// packages/acp-bridge/src/mcpTimeouts.ts
init_esbuild_shims();
var MCP_RESTART_SERVER_DEADLINE_MS = 3e5;

export {
  MCP_RESTART_SERVER_DEADLINE_MS
};
/**
 * @license
 * Copyright 2025 Qwen Team
 * SPDX-License-Identifier: Apache-2.0
 */
