// Force strict mode and setup for ESM
"use strict";
import {
  init_esbuild_shims
} from "./chunk-A4BMJM77.js";

// packages/cli/src/serve/runtime-startup-errors.ts
init_esbuild_shims();
var RUNTIME_STARTUP_CANCELLED_MESSAGE = "Daemon runtime cancelled: server closed before startup.";

export {
  RUNTIME_STARTUP_CANCELLED_MESSAGE
};
/**
 * @license
 * Copyright 2025 Qwen Team
 * SPDX-License-Identifier: Apache-2.0
 */
