// Force strict mode and setup for ESM
"use strict";
import {
  runExitCleanup
} from "./chunk-M7DEWH3V.js";
import {
  init_esbuild_shims
} from "./chunk-A4BMJM77.js";
import {
  __name
} from "./chunk-J2S4EL5Y.js";

// packages/cli/src/utils/processUtils.ts
init_esbuild_shims();
var RELAUNCH_EXIT_CODE = 42;
async function relaunchApp() {
  await runExitCleanup();
  process.exit(RELAUNCH_EXIT_CODE);
}
__name(relaunchApp, "relaunchApp");

export {
  RELAUNCH_EXIT_CODE,
  relaunchApp
};
/**
 * @license
 * Copyright 2025 Google LLC
 * SPDX-License-Identifier: Apache-2.0
 */
